function z = bootstrapconf(nboot,alpha,e,yy,g,h,smoothfun)
%   Compute confidence bands for a non-parametric regression model
%
%   Author: Pieter Jan Kerstens, 2011
%
%   z = BOOTSTRAPCONF(nboot,alpha,e,yy,g,h,smoothfun,sig)
%       nboot: number of bootstraps samples
%       alpha: confidence level
%       e: model errors
%       yy: values generated by the model
%       g,h: kernel bandwiths
%       smoothfun: function handle to the smoother. This function must take
%       2 arguments: Y values to train on and a bandwith g. The function
%       handle is typically of the form: newY = smoothfun(X,Y,newX,h,varargin{:})
%
%       z: quantiles of the empirical bootstrap cumulative distribution
%       matching a confidence level of 100*(1-alpha) percent
%
%   See also: BOOTSTRP, SADDLEPOINT


    % Bootstrap ecdf
    function pb = becdf(x,X)
        pb = zeros(size(x));
        for i=1:length(x)
            pb(i) = length(X(X <= x(i)))/length(X);
        end
    end

    eb = e(:);
    yyb = yy(:);
    
    n = length(e);
    
    yyh = smoothfun(yy,h);
    
    sqnh = sqrt(n*h);
    
    if(isempty(getCurrentJob)) % Are we running on a worker?
        % Check if a worker pool is already open
        isOpen = matlabpool('size') > 0;
        if(~isOpen)
            matlabpool open
        end
    end
    
    options = statset('bootstrp');
    options = statset(options,'UseParallel','always');
    mstarn = bootstrp(nboot,@(t) (smoothfun(t+yyb,g)-yyh).*sqnh,eb,'Options',options);
    
    z1 = zeros(1,n);
    z2 = zeros(1,n);
    becdfpar = @(x,j) becdf(x,mstarn(:,j));
    parfor j=1:n
        z1(j) = fzero(@(x) becdfpar(x,j) - (alpha/2),0);
        z2(j) = fzero(@(x) becdfpar(x,j) - (1-(alpha/2)),0);
    end
    z = [z1;z2];
    
    if(isempty(getCurrentJob))
        % If we opened the worker pool then we close it!
        if(~isOpen)
            matlabpool close
        end
    end
end